'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var graph = require('@algorithm.ts/graph');
var priorityQueue = require('@algorithm.ts/priority-queue');

class Dijkstra {
    INF;
    ZERO;
    bestFrom;
    dist;
    done;
    Q;
    constructor(options = {}) {
        this.INF = options.INF ?? Math.floor(Number.MAX_SAFE_INTEGER / 2);
        this.ZERO = 0;
        this.bestFrom = options.bestFrom ?? [];
        this.dist = options.dist ?? [];
        this.done = [];
        this.Q = priorityQueue.createPriorityQueue((x, y) => {
            if (x.cost === y.cost)
                return 0;
            return x.cost < y.cost ? 1 : -1;
        });
    }
    dijkstra(graph$1, options = {}, onResolved) {
        const { ZERO, Q, done } = this;
        const { N, source, edges, G } = graph$1;
        const { INF = this.INF, bestFrom = this.bestFrom, dist = this.dist } = options;
        dist.length = N;
        done.length = N;
        dist.fill(INF, 0, N);
        done.fill(false, 0, N);
        Q.init();
        Q.enqueue({ pos: source, cost: ZERO });
        dist[source] = ZERO;
        while (Q.size() > 0) {
            const { pos: o, cost } = Q.dequeue();
            if (done[o]) {
                if (dist[o] < cost)
                    return false;
                continue;
            }
            done[o] = true;
            for (const idx of G[o]) {
                const edge = edges[idx];
                const candidate = dist[o] + edge.cost;
                if (dist[edge.to] > candidate) {
                    dist[edge.to] = candidate;
                    bestFrom[edge.to] = o;
                    Q.enqueue({ pos: edge.to, cost: dist[edge.to] });
                }
            }
        }
        if (onResolved) {
            const getShortestPathTo = (target) => graph.getShortestPath(bestFrom, source, target);
            const context = {
                INF: this.INF,
                bestFrom: this.bestFrom,
                dist: this.dist,
                getShortestPathTo,
            };
            onResolved(context);
        }
        return true;
    }
}

function dijkstra(graph, options = {}, onResolved) {
    if (options.dist === undefined)
        options.dist = [];
    _dijkstra.dijkstra(graph, options, onResolved);
    return options.dist;
}
const _dijkstra = new Dijkstra();

exports.Dijkstra = Dijkstra;
exports["default"] = dijkstra;
exports.dijkstra = dijkstra;
