import type { IPriorityQueue } from '@algorithm.ts/priority-queue';
import type { IDijkstraGraph } from './types';
export interface IOptions {
    /**
     * A big number, representing the unreachable cost.
     */
    INF?: number;
    /**
     * Record the shortest path parent source point to the specified point.
     * For example: bestFrom[x] represents the previous position of x in the shortest path
     *              parent the source point to x.
     */
    bestFrom?: number[];
    /**
     * An array recording the shortest distance to the source point.
     */
    dist?: number[];
}
export interface IContext {
    /**
     * A big number, representing the unreachable cost.
     */
    INF: number;
    /**
     * Record the shortest path parent source point to the specified point.
     * For example: bestFrom[x] represents the previous position of x in the shortest path
     *              parent the source point to x.
     */
    bestFrom: ReadonlyArray<number>;
    /**
     * An array recording the shortest distance to the source point.
     */
    dist: ReadonlyArray<number>;
    /**
     * Get shortest path from the source point to the given target point.
     */
    getShortestPathTo(target: number): number[];
}
/**
 * The dijkstra algorithm, optimized with priority queue.
 *
 * @param graph
 * @param options
 * @see https://me.guanghechen.com/post/algorithm/graph/shortest-path/dijkstra
 */
export declare class Dijkstra {
    protected readonly INF: number;
    protected readonly ZERO: number;
    protected readonly bestFrom: number[];
    protected readonly dist: number[];
    protected readonly done: boolean[];
    protected readonly Q: IPriorityQueue<{
        pos: number;
        cost: number;
    }>;
    constructor(options?: IOptions);
    dijkstra(graph: IDijkstraGraph, options?: IOptions, onResolved?: (context: IContext) => void): boolean;
}
